@@ -1,4 +1,4 @@
-/* $Id: xfsm-startup.c 28617 2008-11-06 06:50:53Z kelnos $ */
+/* $Id$ */
 /*-
  * Copyright (c) 2003-2006 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2008 Brian Tarricone <bjt23@cornell.edu>
@@ -46,7 +46,7 @@
 
 #include <glib/gstdio.h>
 
-#include <libxfcegui4/libxfcegui4.h>
+#include <libxfce4ui/libxfce4ui.h>
 
 #include <libxfsm/xfsm-util.h>
 
@@ -263,7 +263,6 @@ xfsm_startup_autostart_migrate (void)
                                "Terminal=False\n"
                                "StartupNotify=False\n"
                                "Version=0.9.4\n"
-                               "Encoding=UTF-8\n"
                                "Name=%s\n",
                                entry, target_path, entry);
                       fclose (fp);
@@ -304,13 +303,15 @@ xfsm_startup_autostart_migrate (void)
 
 
 static gint
-xfsm_startup_autostart_xdg (void)
+xfsm_startup_autostart_xdg (XfsmManager *manager)
 {
   const gchar *try_exec;
   const gchar *type;
   const gchar *exec;
   gboolean     startup_notify;
   gboolean     terminal;
+  gboolean     gnome;
+  gboolean     kde;
   gboolean     skip;
   GError      *error = NULL;
   XfceRc      *rc;
@@ -323,6 +324,10 @@ xfsm_startup_autostart_xdg (void)
   /* migrate the old autostart location (if still present) */
   xfsm_startup_autostart_migrate ();
 
+  /* Get if we should start KDE and GNOME applications */
+  kde = xfsm_manager_get_compat_startup (manager, XFSM_MANAGER_COMPAT_KDE);
+  gnome = xfsm_manager_get_compat_startup (manager, XFSM_MANAGER_COMPAT_GNOME);
+
   files = xfce_resource_match (XFCE_RESOURCE_CONFIG, "autostart/*.desktop", TRUE);
   for (n = 0; files[n] != NULL; ++n)
     {
@@ -340,13 +345,21 @@ xfsm_startup_autostart_xdg (void)
           only_show_in = xfce_rc_read_list_entry (rc, "OnlyShowIn", ";");
           if (G_UNLIKELY (only_show_in != NULL))
             {
-              /* check if "Xfce" is specified */
+              /* check if "XFCE" is specified */
+              /* If we start the GNOME components, also start the applications when
+               * then "GNOME" is specified. */
+              /* If we start the KDE components, also start the applications when
+               * then "KDE" is specified. */
               for (m = 0, skip = TRUE; only_show_in[m] != NULL; ++m)
-                if (g_ascii_strcasecmp (only_show_in[m], "Xfce") == 0)
-                  {
-                    skip = FALSE;
-                    break;
-                  }
+                {
+                  if ((g_ascii_strcasecmp (only_show_in[m], "XFCE") == 0) ||
+                      (gnome && g_ascii_strcasecmp (only_show_in[m], "GNOME") == 0) ||
+                      (kde && g_ascii_strcasecmp (only_show_in[m], "KDE") == 0))
+                    {
+                      skip = FALSE;
+                      break;
+                    }
+                }
 
               g_strfreev (only_show_in);
             }
@@ -388,7 +401,12 @@ xfsm_startup_autostart_xdg (void)
           terminal = xfce_rc_read_bool_entry (rc, "Terminal", FALSE);
 
           /* try to launch the command */
-          if (!xfce_exec (exec, terminal, startup_notify, &error))
+          xfsm_verbose ("Autostart: running command \"%s\"\n", exec);
+          if (!xfce_spawn_command_line_on_screen (gdk_screen_get_default (),
+                                                  exec,
+                                                  terminal,
+                                                  startup_notify,
+                                                  &error))
             {
               g_warning ("Unable to launch \"%s\" (specified by %s): %s", exec, files[n], error->message);
               g_error_free (error);
@@ -411,11 +429,11 @@ xfsm_startup_autostart_xdg (void)
 
 
 static void
-xfsm_startup_autostart (void)
+xfsm_startup_autostart (XfsmManager *manager)
 {
   gint n;
 
-  n = xfsm_startup_autostart_xdg ();
+  n = xfsm_startup_autostart_xdg (manager);
 
   if (n > 0)
     {
@@ -449,7 +467,8 @@ xfsm_startup_begin (XfsmManager *manager)
   if (xfsm_manager_get_use_failsafe_mode (manager))
     {
       xfsm_startup_failsafe (manager);
-      xfsm_startup_autostart ();
+      xfsm_startup_autostart (manager);
+      xfsm_manager_signal_startup_done (manager);
     }
   else
     {
@@ -488,26 +507,31 @@ xfsm_startup_start_properties (XfsmProperties *properties,
 {
   XfsmStartupData *child_watch_data;
   XfsmStartupData *startup_timeout_data;
+  gchar          **restart_command;
   gchar          **argv;
   gint             argc;
   gint             n;
+  const gchar     *current_directory;
   GPid             pid;
 
   /* release any possible old resources related to a previous startup */
   xfsm_properties_set_default_child_watch (properties);
 
   /* generate the argument vector for the application (expanding variables) */
-  argc = g_strv_length (properties->restart_command);
+  restart_command = xfsm_properties_get_strv (properties, SmRestartCommand);
+  argc = g_strv_length (restart_command);
   argv = g_new (gchar *, argc + 1);
   for (n = 0; n < argc; ++n)
-    argv[n] = xfce_expand_variables (properties->restart_command[n], NULL);
+    argv[n] = xfce_expand_variables (restart_command[n], NULL);
   argv[n] = NULL;
 
+  current_directory = xfsm_properties_get_string (properties, SmCurrentDirectory);
+
   /* fork a new process for the application */
 #ifdef HAVE_VFORK
   /* vfork() doesn't allow you to do anything but call exec*() or _exit(),
    * so if we need to set the working directory, we can't use vfork() */
-  if (properties->current_directory == NULL)
+  if (current_directory == NULL)
     pid = vfork ();
   else
 #endif
@@ -517,8 +541,11 @@ xfsm_startup_start_properties (XfsmProperties *properties,
   if (pid == 0)
     {
       /* execute the application here */
-      if (properties->current_directory)
-        chdir (properties->current_directory);
+      if (current_directory)
+        {
+          if (chdir (current_directory))
+            g_warning ("Unable to chdir to \"%s\": %s", current_directory, strerror (errno));
+        }
       execvp (argv[0], argv);
       _exit (127);
     }
@@ -577,7 +604,7 @@ xfsm_startup_session_continue (XfsmManager *manager)
       /* we failed to start anything, and we don't have anything else,
        * to start, so just move on to the autostart items and signal
        * the manager that we're finished */
-      xfsm_startup_autostart ();
+      xfsm_startup_autostart (manager);
       xfsm_manager_signal_startup_done (manager);
     }
 }
@@ -597,14 +624,14 @@ xfsm_startup_session_next_prio_group (XfsmManager *manager)
   if (properties == NULL)
     return FALSE;
 
-  cur_prio_group = properties->priority;
+  cur_prio_group = xfsm_properties_get_uchar (properties, GsmPriority, 50);
 
   xfsm_verbose ("Starting apps in prio group %d\n", cur_prio_group);
 
   while ((properties = g_queue_pop_head (pending_properties)))
     {
       /* quit if we've hit all the clients in the current prio group */
-      if (properties->priority != cur_prio_group)
+      if (xfsm_properties_get_uchar (properties, GsmPriority, 50) != cur_prio_group)
         {
           /* we're not starting this one yet; put it back */
           g_queue_push_head (pending_properties, properties);
@@ -614,8 +641,34 @@ xfsm_startup_session_next_prio_group (XfsmManager *manager)
       /* FIXME: splash */
       if (G_LIKELY (splash_screen != NULL))
         {
-          xfsm_splash_screen_next (splash_screen,
-                                   figure_app_name (properties->program));
+          const gchar *app_name = NULL;
+          const gchar *desktop_file;
+          XfceRc      *rcfile = NULL;
+
+          desktop_file = xfsm_properties_get_string (properties, GsmDesktopFile);
+
+          if (desktop_file)
+            {
+              rcfile = xfce_rc_simple_open (desktop_file, TRUE);
+              if (rcfile)
+                {
+                  xfce_rc_set_group (rcfile, "Desktop Entry");
+                  app_name = xfce_rc_read_entry (rcfile, "Name", NULL);
+                }
+            }
+
+          if (!app_name)
+            app_name = figure_app_name (xfsm_properties_get_string (properties,
+                                                                    SmProgram));
+
+          xfsm_splash_screen_next (splash_screen, app_name);
+
+          if (rcfile)
+            {
+              /* delay closing because app_name belongs to the rcfile
+               * if we found it in the file */
+              xfce_rc_close (rcfile);
+            }
         }
 
       if (G_LIKELY (xfsm_startup_start_properties (properties, manager)))
@@ -654,6 +707,9 @@ xfsm_startup_child_watch (GPid     pid,
   xfsm_verbose ("Client Id = %s, PID %d exited with status %d\n",
                 cwdata->properties->client_id, (gint)pid, status);
 
+  cwdata->properties->child_watch_id = 0;
+  cwdata->properties->pid = -1;
+
   starting_properties = xfsm_manager_get_queue (cwdata->manager, XFSM_MANAGER_QUEUE_STARTING_PROPS);
   if (g_queue_find (starting_properties, cwdata->properties) != NULL)
     {
@@ -662,8 +718,9 @@ xfsm_startup_child_watch (GPid     pid,
       xfsm_startup_handle_failed_startup (cwdata->properties, cwdata->manager);
     }
 
-  cwdata->properties->child_watch_id = 0;
-  cwdata->properties->pid = -1;
+  /* NOTE: cwdata->properties could have been freed by
+   * xfsm_startup_handle_failed_startup() above, so don't try to access
+   * any of its members here. */
 
   g_spawn_close_pid (pid);
 }
